--[[-----------------------------------------------------------------------------
 Copyright (c) 2012 Sierra Wireless and others.
 All rights reserved. This program and the accompanying materials
 are made available under the terms of the Eclipse Public License v1.0
 which accompanies this distribution, and is available at
 http://www.eclipse.org/legal/epl-v10.html

 Contributors:
     Gilles Cannenterre for Sierra Wireless - initial API and implementation
-----------------------------------------------------------------------------
]]

--[[------------------------------------------------------------------------------
 Implementation of modbus TCP, TCP/RTU and TCP/ASCII.

 This module relies on the @{socket#socket} module to handle socket configuration and
 actual data exchanges.

 Data read and written by this module are exchanged as _buffers_, i.e. either
 strings or lists of strings. String(s) contain data as 8-bits bytes;
 endianness depends on what is expected/returned by the slave device.
 To encode such strings, it is suggested to rely on @{pack#pack}, the binary
 string packing library.

 @module modbustcp

]]

--[[------------------------------------------------------------------------------
 Creates a new @{#modbusdev}.
 Default initialization is `TCP`

 @function [parent=#modbus] new
 @param cfg, an optional @{#config} table.
  Default values are
  `{ maxsocket = 1, timeout = 1 }`
 @param mode tcp mode as a string: `"TCP", "ASCII"` or `"RTU"`. <br />
 Defaults to `"TCP"`.
 @return #modbusdev the new @{#modbusdev} on success
 @return `nil` + error message

]]

--[[------------------------------------------------------------------------------
 Modbus configuration table.

 @type config
 @field maxsocket
 to set the maximum number of socket in use.
 accepted value is a stricly positive integer.
 @field timeout
 to configure the socket timeout, in seconds.
 accepted value is a stricly positive integer.

]]

--[[------------------------------------------------------------------------------
 Closes a modbus instance.

 @function [parent=#modbusdev] close
 @param self
 @return "ok"

]]

--[[------------------------------------------------------------------------------
 Read a sequence of consecutive coils' content.

 @function [parent=#modbusdev] readCoils
 @param self
 @param host ip address or resolvable hostname of server
 @param port port number of server, defaults to port 502
 @param sid number defining the slave id to send the request to
 @param address starting address
 @param length number of coils
 @return read coil values in a buffer
 @return `nil` + error message

]]

--[[---------------------------------------------------------------------------------
 Read a sequence of consecutive discrete inputs' content.

 @function [parent=#modbusdev] readDiscreteInputs
 @param self
 @param host ip address or resolvable hostname of server
 @param port port number of server, defaults to port 502
 @param sid number defining the slave id to send the request to
 @param address starting address
 @param length number of inputs
 @return read discrete inputs contant in a buffer
 @return `nil` + error message

]]

--[[---------------------------------------------------------------------------------
 Read a sequence of consecutive holding registers' content.

 @function [parent=#modbusdev] readHoldingRegisters
 @param self
 @param host ip address or resolvable hostname of server
 @param port port number of server, defaults to port 502
 @param sid number defining the slave id to send the request to
 @param address starting address
 @param length number of registers
 @return read holding registers content in a buffer
 @return `nil` + error message

]]

--[[---------------------------------------------------------------------------------
 Read a sequence of consecutive input registers' content.

 @function [parent=#modbusdev] readInputRegisters
 @param self
 @param host ip address or resolvable hostname of server
 @param port port number of server, defaults to port 502
 @param sid number defining the slave id to send the request to
 @param address starting address
 @param length number of inputs
 @return read input registers content in a buffer
 @return `nil` + error message

]]

--[[---------------------------------------------------------------------------------
 Write a value in a coil.

 @function [parent=#modbusdev] writeSingleCoil
 @param self
 @param host ip address or resolvable hostname of server
 @param port port number of server, defaults to port 502
 @param sid number defining the slave id to send the request to
 @param address address of the coil
 @param value Boolean to write in the coil
 @return `"ok"`
 @return `nil` + error message

]]

--[[---------------------------------------------------------------------------------
 Write a value in a register.

 @function [parent=#modbusdev] writeSingleRegister
 @param self
 @param host ip address or resolvable hostname of server
 @param port port number of server, defaults to port 502
 @param sid number defining the slave id to send the request to
 @param address address of the coil
 @param value Short integer (`[0..0xFFFF]`) to write in the register
 @return `"ok"`
 @return `nil` + error message

]]

--[[---------------------------------------------------------------------------------
 Write a sequence of consecutive coils.

 @function [parent=#modbusdev] writeMultipleCoils
 @param self
 @param host ip address or resolvable hostname of server
 @param port port number of server, defaults to port 502
 @param sid number defining the slave id to send the request to
 @param address starting address
 @param nbvalues number of coils to write
 @param values values to write, as a bytes buffer (8 coil per byte)
 @return `"ok"`
 @return `nil` + error message
 @usage
  m = modbustcp.new()
  res,err=m:writeMultipleCoils('10.41.51.50', 502, 1, 0, 8, string.pack('x8',
  true,false,false,true,false,true,false,true)

]]

--[[---------------------------------------------------------------------------------
 Write a sequence of consecutive registers.

 @function [parent=#modbusdev] writeMultipleRegisters
 @param self
 @param host ip address or resolvable hostname of server
 @param port port number of server, defaults to port 502
 @param sid number defining the slave id to send the request to
 @param address starting address
 @param values values to write, as a word buffer (each register is a 16bits word)
 @return `"ok"`
 @return `nil` + error message
 @usage
  m = modbustcp.new()
  res,err=m:writeMultipleRegisters('10.41.51.50', 502, 1, 0, string.pack("<H8",21,159,357,654,852,
  357,654,852))

]]

--[[-----------------------------------------------------------------------------
 Send a custom MODBUS request, not directly supported by the API.

 @function [parent=#modbusdev] customRequest
 @param self
 @param req number identifying the custom function
 @param host ip address or resolvable hostname of server
 @param port port number of server, defaults to port 502
 @param sid slave id
 @param payload optional data bytes to send, as a string
 @return the response as a buffer.
   Beware, it contains some error-checking bytes as a suffix.
 @return `nil` + error message.

]]

return nil
