--[[-----------------------------------------------------------------------------
 Copyright (c) 2012 Sierra Wireless and others.
 All rights reserved. This program and the accompanying materials
 are made available under the terms of the Eclipse Public License v1.0
 which accompanies this distribution, and is available at
 http://www.eclipse.org/legal/epl-v10.html

 Contributors:
     Laurent Barthelemy for Sierra Wireless - initial API and implementation
     Cuero Bugot        for Sierra Wireless - initial API and implementation
-----------------------------------------------------------------------------
]]

--[[-
 Table utils.

 @module utils.table

]]

--[[------------------------------------------------------------------------------
 Packs a variable number of arguments into a table.
 This is a temporary function until we switch to Lua 5.2.

 Returns a new table with all parameters stored into keys 1, 2, etc. and with a field "n" with
 the total number of parameters. Also returns, as a second result, the total number of parameters.

 Note that the resulting table may not be a sequence.

 Note that when loading utils.table module, this pack function is also added to global/std table module.

 @function [parent=#utils.table] pack
 @param varargs arguments to pack
 @return a table that contains all the arguments followed by the number of arguments.

]]

--[[------------------------------------------------------------------------------
 Copies a table from the source to destination table.

 The copy is recursive.

 @function [parent=#utils.table] copy
 @param src a table to be used as the source of the copy.
 @param dst a table to be used as the destination of the copy.
 @param overwrite boolean value, to enable overwrite of existing field in dst table.
 @return the resulting dst table.

]]

--[[------------------------------------------------------------------------------
 Produces a diff of two tables.

 Recursive diff by default.

 @function [parent=#utils.table] diff
 @param t1 first table to compare.
 @param t2 second table to compare.
 @param norecurse boolean value to disable recursive diff.
 @return the diff result as a table.
 @usage
> t1 = { a = 1, b=3, c = "foo"}
> t2 = { a = 1, b=3, c = "foo"}
> :diff(t1, t2)
= {  }
> t1 = { a = 1, b=3, c = "foo"}
> t2 = { a = 1, b=4, c = "foo"}
> :diff(t1, t2)
= { "b" }
> t1 = { a = 1, b=3, c = "foo"}
> t2 = { a = 1, b=3}
> :diff(t1, t2)
= { "c" }
> t1 = { b=3, c = "foo", d="bar"}
> t2 = { a = 1, b=3, c = "foo"}
> :diff(t1, t2)
= {
   "a",
   "d" }

]]

--[[------------------------------------------------------------------------------
 Checks if the table is regular array.

 Regular array is a table with only conscutive integer keys (no holes).

 @function [parent=#utils.table] isArray
 @param T table to check.
 @return true or false.

]]

--[[------------------------------------------------------------------------------
 Extracts the keys of a table into a list.

 @function [parent=#utils.table] keys
 @param T the table to extract keys from.
 @return a list of keys of T table.

]]

--[[------------------------------------------------------------------------------
 Executes a function on each element of a table.

 If recursive is non false the map function will be called recursively.

 If inplace is set to true, then the table is modified in place.

 The function is called with key and value as a parameter.

 @function [parent=#utils.table] map
 @param T the table to apply map on.
 @param func the function to call for each value of the map.
 @param recursive if non false, the map is executed recusively.
 @param inplace if non false, the table is modified in place.
 @return the table with each value mapped by the function
 @usage local data = { key1 = "val1", key2 = "val2"}
 local function foo(key, value) print("foo:" key, value) end
 map(data, foo)
 -> will print:
 foo:    key1        val1
 foo:    key2        val2

]]

--[[- Iterators
 @section It
]]

--[[------------------------------------------------------------------------------
 Iterator that iterates on the table in key ascending order.

 @function [parent=#utils.table] sortedPairs
 @param t table to iterate.
 @return iterator function.

]]

--[[------------------------------------------------------------------------------
 Iterator that returns key/value pairs except walking through sub tables and concatenating the path in the key.

 This iterator break cycles: it does not recur. If it detects a cycle, the entry causing a cycle is ignored.
 However, if a table is repeated, its contents will be repeated by this function.

 @function [parent=#utils.table] recursivePairs
 @param t table to iterate.
 @param prefix path prefix to prepend to the key path returned.
 @return iterator function.
 @usage {toto={titi=1, tutu=2}, tata = 3, tonton={4, 5}} will iterate through
 ("toto.titi",1), ("toto.tutu",2), ("tata",3) ("tonton.1", 4), ("tonton.2"=5)

]]

--[[------------------------------------------------------------------------------
 Iterator that returns key/value on multiple tables.

 @function [parent=#utils.map] multiPairs
 Tables are traversed in the same order as they are passed as parameters.
 @param t1 first table to traverse.
 @param ... additional tables to traverse.
 @return iterator function.

]]

return nil
